'use client';
import { useEffect, useId, useRef, useState } from 'react';
import { Send, Trash } from 'react-feather';
import toast from 'react-hot-toast';
import { createMessageFileAudio } from "@/feature/messenger/services/chat/ChatService";
import useMessagesStore from "@/feature/messenger/store/chat_store/useMessagesStore";
import useConversationsStore from "@/feature/messenger/store/chat_store/useConversationsStore";
import useConfigChatStore from "@/feature/messenger/store/chat_store/useConfigChatStore";
import { Button, Tooltip } from '@nextui-org/react';
import { useTranslation } from '@/hooks/useTranslation';
import useResponsiveBtn from '@/feature/messenger/hooks/useReponsiveBtn';


const RecordAudio = ({ handleClearRecord }) => {
    const id_chatbot = useConfigChatStore(state => state.idChatbot);
    const selectedConversation = useConversationsStore(state => state.selectedConversation);
    const addItemListMessages = useMessagesStore(state => state.actions.addItemListMessages);
    const updateConversation = useConversationsStore(state => state.actions.updateConversation);
    const mimeType = "audio/mp3";
    const [time, setTime] = useState({ minutes: 0, seconds: 0 });
    const [timeStop, setTimeStop] = useState({ minutes: 0, seconds: 0 });
    const [permission, setPermission] = useState(false);
    const [stream, setStream] = useState(null);
    const mediaRecorder = useRef(null);
    const [recordingStatus, setRecordingStatus] = useState("inactive");
    const [audioChunks, setAudioChunks] = useState([]);
    const [audio, setAudio] = useState(null);
    const buttonStartRef = useRef(null);
    const [loading, setLoading] = useState(false);
    const { trans } = useTranslation();
    const { responsiveButtonNoColorFooter, responsiveIconButtonFooter } = useResponsiveBtn();

    useEffect(() => {
        if (permission) {
            if (buttonStartRef.current) {
                buttonStartRef.current.click();
            }
        }

        const intervalId = setInterval(() => {
            // Incrementar los segundos
            if (permission && recordingStatus === "recording") {
                setTime(prevTime => {
                    let updatedSeconds = prevTime.seconds + 1;
                    let updatedMinutes = prevTime.minutes;

                    // Verificar si hay que incrementar los minutos
                    if (updatedSeconds === 60) {
                        updatedSeconds = 0;
                        updatedMinutes++;
                    }

                    return { minutes: updatedMinutes, seconds: updatedSeconds };
                });
            }
        }, 1000); // Actualizar cada segundo

        return () => clearInterval(intervalId);
    }, [permission, recordingStatus]);

    const blobToFile = (blob, fileName) => {
        const file = new File([blob], fileName);
        return file;
    };
    const stopRecording = () => {
        setPermission(false);
        setLoading(true);
        setTimeStop({ minutes: time.minutes, seconds: time.seconds });
        if (!mediaRecorder.current) {
            return;
        }
        setRecordingStatus("inactive");
        //stops the recording instance
        mediaRecorder.current.stop();
        mediaRecorder.current.onstop = () => {
            //creates a blob file from the audiochunks data
            const audioBlob = new Blob(audioChunks, { type: mimeType });
            //creates a playable URL from the blob file.
            const audioUrl = URL.createObjectURL(audioBlob);
            setAudio(audioUrl);
            setAudioChunks([]);
            const file = new File([audioBlob], "audio.mp3", {
                type: 'audio/mp3'
            });

            createMessageFileAudio(id_chatbot, selectedConversation.sesion, file).then((res) => {

                if (res.data === null) {
                    toast.error("Error al enviar el audio");
                } else {
                    const response = res.data;
                    for (const iterator of response) {
                        addItemListMessages(iterator);
                        const currentSesion = selectedConversation;
                        currentSesion.conversacion = iterator.conversacion;
                        currentSesion.tipomensaje = iterator.tipomensaje;
                        currentSesion.fecha = iterator.fecha_sistema;
                        currentSesion.proceso = iterator.proceso;
                        updateConversation(currentSesion);

                    }
                    toast.success("Audio enviado correctamente", {
                        autoClose: 5000,
                        position: 'top-center'
                    });
                }

                handleClearRecord();
                setLoading(false);
            });
        };
    };

    const startRecording = async () => {
        setRecordingStatus("recording");
        //create new Media recorder instance using the stream
        const media = new MediaRecorder(stream, { type: mimeType });
        //set the MediaRecorder instance to the mediaRecorder ref
        mediaRecorder.current = media;
        //invokes the start method to start the recording process
        mediaRecorder.current.start();
        let localAudioChunks = [];
        mediaRecorder.current.ondataavailable = (event) => {
            if (typeof event.data === "undefined") return;
            if (event.data.size === 0) return;
            localAudioChunks.push(event.data);
        };
        setAudioChunks(localAudioChunks);
    };

    useEffect(() => {
        const getMicrophonePermission = async () => {
            if ("MediaRecorder" in window) {
                try {
                    const streamData = await navigator.mediaDevices.getUserMedia({
                        audio: true,
                        video: false,
                    });
                    setPermission(true);
                    setStream(streamData);
                } catch (err) {
                    toast.error(err.message);
                    handleClearRecord();
                }
            } else {
                toast.error("The MediaRecorder API is not supported in your browser.");
                handleClearRecord();
            }
        };

        getMicrophonePermission();

    }, []);
    const handleStopClearRecord = () => {
        handleClearRecord();
    }

    const handleSendRecord = () => {
        stopRecording();
    }

    const timeRecord = () => {
        if (permission && recordingStatus === "recording") {
            return (
                <>
                    {time.minutes < 10 ? `0${time.minutes}` : time.minutes}:{time.seconds < 10 ? `0${time.seconds}` : time.seconds}
                </>
            )
        } else {
            return (
                <>
                    {timeStop.minutes < 10 ? `0${timeStop.minutes}` : timeStop.minutes}:{timeStop.seconds < 10 ? `0${timeStop.seconds}` : timeStop.seconds}
                </>
            )
        }
    }
    return (
        <>
            <div className="flex flex-row justify-center items-center gap-2">
                {/* Time Record */}
                <div className='flex justify-center items-center gap-1'>
                    <button ref={buttonStartRef} onClick={() => startRecording()} style={{ display: 'none' }}></button>
                    <div style={{ display: 'none' }} className="audio-container">
                        <audio src={audio} controls></audio>
                        <a download href={audio}>
                            Download Recording
                        </a>
                    </div>
                    <div className="w-4 h-4 bg-red-500 rounded-full animate-blink"></div>
                    {timeRecord()}
                </div>
                {/* Clear Record */}
                <div>
                    <Tooltip
                        placement="top"
                        content={trans('footer_content_chat_stop_button_title')}	
                        color="default"
                    >
                        <Button
                            className={responsiveButtonNoColorFooter}
                            onClick={() => handleStopClearRecord()}
                            isDisabled={loading}
                            color='danger'
                            radius='full'
                        >
                            <Trash className={responsiveIconButtonFooter} />
                        </Button>
                    </Tooltip>
                </div>
                {/* Send Record */}
                <div>
                    <Tooltip
                        placement="top"
                        content={trans('footer_content_chat_send_mic_button_title')}	
                        color="default"
                    >
                        <Button
                            onClick={() => handleSendRecord()}
                            className={responsiveButtonNoColorFooter}
                            isDisabled={loading}
                            color='primary'
                            radius='full'
                        >
                            <Send className={responsiveIconButtonFooter} />
                        </Button>
                    </Tooltip>
                </div>
            </div>
        </>
    )
}

export default RecordAudio